#
# Copyright (c) 2010, 2013
#  Adam Tauno Williams <awilliam@whitemice.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE TASK
#
from coils.foundation import Task, apply_orm_hints_to_query
#from coils.core import *
from coils.core.logic import SearchCommand
from keymap import COILS_TASK_KEYMAP


class SearchTasks(SearchCommand):
    # TODO: needs notations support
    __domain__ = "task"
    __operation__ = "search"
    mode = None

    def __init__(self):
        SearchCommand.__init__(self)

    def prepare(self, ctx, **params):
        SearchCommand.prepare(self, ctx, **params)

    def parse_parameters(self, **params):
        SearchCommand.parse_parameters(self, **params)

    def add_result(self, task):
        if (task not in self._result):
            self._result.append(task)

    @property
    def search_keymap(self):
        keymap = COILS_TASK_KEYMAP.copy()
        keymap.update(
            {'ownerid':         ['owner_id', 'int', None, ],
             'owner_id':        ['owner_id', 'int', None, ],
             'ownerobjectid':   ['owner_id', 'int', None, ],
             'status':          ['state', 'str', None, ],
             'modified':        ['modified', 'date', None, ], }
        )
        return keymap

    def run(self):

        keymap = self.search_keymap
        self._query = \
            self._parse_criteria(self._criteria, Task, keymap, )
        self._query = \
            self._apply_sort_criteria(
                query=self._query,
                entity=Task,
                keymap=keymap,
                sort_keys=self._sort_keys,
                reverse_order=self._reverse_sort,
            )
        self._query = \
            apply_orm_hints_to_query(
                self._query, Task, self.orm_hints,
            )
        data = self._query.all()
        self.log.debug('query returned %d objects' % len(data))
        self.set_return_value(data)
        return
