#
# Copyright (c) 2009, 2014
#  Adam Tauno Williams <awilliam@whitemice.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE
#
from coils.core.logic import ActionCommand


class WriteAction(ActionCommand):
    __domain__ = "action"
    __operation__ = "write"
    __aliases__ = ['writeAction', ]

    def __init__(self):
        ActionCommand.__init__(self)

    def parse_action_parameters(self):
        self._format_name = self.action_parameters.get('format', 'StandardRaw')

    @property
    def result_mimetype(self):
        return self._format.mimetype

    def do_action(self):
        self._format = self._ctx.run_command(
            'format::get', name=self._format_name,
        )

        self.log_message(
            'Marshalled format "{0}"'.format(self._format_name, ),
            category='info',
        )

        self._format.process_out(self.rfile, self.wfile)
        self.wfile.flush()

        """
        Not all formats may support record counting, in which case this call
        should return a None, and nothing is recorded.
        """
        counter = self._format.get_record_counter()
        if counter is None:
            self.log_message(
                'Class of format "{0}" does not implement count reporting.'
                .format(self._format_name, )
            )
            return

        self.log_message(
            '{0} records processes by format write'
            .format(counter, ),
            category='info',
        )
        self._ctx.property_manager.set_property(
            entity=self.process,
            namespace='http://www.opengroupware.us/oie',
            attribute='lastFormatWriteCount',
            value=counter,
        )
