#
# Copyright (c) 2010, 2012, 2013, 2014, 2015, 2016, 2017
#  Adam Tauno Williams <awilliam@whitemice.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
from time import sleep
from coils.core import CoilsException
from coils.core.logic import ActionCommand
from utility import sql_connect
from command import SQLCommand


class InsertAction(ActionCommand, SQLCommand):
    # TODO: Needs doCommit support.
    __domain__ = "action"
    __operation__ = "sql-insert"
    __aliases__ = ['sqlInsert', 'sqlInsertAction', ]

    def __init__(self):
        ActionCommand.__init__(self)

    def do_action(self):

        db = sql_connect(self._source)
        cursor = db.cursor()

        """prequery support"""
        if self._prequery:
            try:
                cursor.execute(self._prequery)
            except Exception as exc:
                self.log_message(
                    'Exception performing SQL Prequery: {0}'
                    .format(self._prequery, ), category='error',
                )
                raise exc
            else:
                self.log_message(
                    'Performed SQL Prequery: {0}'.format(self._prequery, ),
                    category='info',
                )
        else:
            self.log_message('No prequery to perform.', category='info', )

        self.log.debug('Reading metadata from input')
        table_name, format_name, format_class, = \
            self._read_result_metadata(self.rfile)
        '''
        If a table was specified via a parameter, that overrides the table
        specified in the StandardXML stream.
        '''
        if self._target:
            table_name = self._target
            self.log_message(
                'Table is "{0}", from action parameter stream'.
                format(table_name, ), category='info',
            )
        else:
            self.log_message(
                'Table is "{0}", from StandardXML stream'.
                format(table_name, ), category='info',
            )

        counter = 0
        try:
            row_count = 0
            for keys, fields in (
                self._read_rows(
                    self.rfile,
                    key_fields=self._keys,
                    for_operation='insert',
                    for_record_type=self._record_type,
                )
            ):
                sql, values = \
                    self._create_insert_from_fields(
                        db, table_name, keys, fields,
                    )
                try:
                    cursor.execute(sql, values)
                    row_count += 1
                except Exception as exc:
                    message = (
                        'error: FAILED SQL: {0} VALUES: {1}'.
                        format(unicode(sql), unicode(values), )
                    )
                    self.log_message(message, category='error', )
                    raise exc
                counter += 1
                if not (counter % 8192):
                    self.gong()
                    sleep(0.1)
            self.log_message(
                '{0} records inserted to SQL connection'.
                format(row_count), category='info',
            )
            self._ctx.property_manager.set_property(
                entity=self.process,
                namespace='http://www.opengroupware.us/oie',
                attribute='sqlLastInsertCount',
                value=row_count,
            )
        finally:
            cursor.close()
            db.close()

    def parse_action_parameters(self):

        """initializes dataSource and preQuery"""
        SQLCommand.parse_action_parameters(self)

        # tableName
        self._target = self.action_parameters.get('tableName', None)
        if self._target:
            self._target = self.process_label_substitutions(self._target)

        # primaryKeys
        self._keys = self.action_parameters.get('primaryKeys', None)
        if self._keys:
            self._keys = self.process_label_substitutions(self._keys)
        if isinstance(self._keys, basestring):
            self._keys = self._keys.split(',')
        else:
            self._keys = list()

        if self._source is None:
            raise CoilsException('No source defined for selectAction')

    def do_epilogue(self):
        pass
