#
# Copyright (c) 2010, 2015 Adam Tauno Williams <awilliam@whitemice.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
import logging
from coils.core import AnonymousContext, NotImplementedException

class OpenGroupwareServerSideFilter(object):

    def __init__(self, rfile, mimetype, parameters, log=None, ctx=None):
        self._rfile = rfile
        for key in parameters:
            setattr(self, '_{0}'.format(key), parameters[key])
        self._mimetype = mimetype
        self._log = log
        self._ctx = ctx

    @property
    def rfile(self):
        return self._rfile

    @property
    def log(self):
        if (self._log is None):
            self._log = logging.getLogger('ossf')
        return self._log

    @property
    def ctx(self):
        if (self._ctx is None):
            self._ctx = AnonymousContext()
        return self._ctx

    @property
    def handle(self):
        raise NotImplementedException('This OSSF is not implemented.')

    @property
    def mimetype(self):
        raise NotImplementedException('This OSSF is not implemented.')
