#
# Copyright (c) 2012, 2014
#  Adam Tauno Williams <awilliam@whitemice.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
from coils.core import NotImplementedException
from coils.net import Protocol, PathObject
from namedpathobject import NamedPathObject
from opengroupware import OpenGroupwareFolder


class RedirectObject(NamedPathObject):

    def __init__(self, parent, name, **params):
        NamedPathObject.__init__(self, parent, name, **params)

    def do_GET(self):
        self.request.simple_response(
            301, headers={'Location': self.location, },
        )

    def do_PROPFIND(self):
        self.request.simple_response(
            301, headers={'Location': self.location, },
        )

WELL_KNOWN_REDIRECTS = {
    'carddav': '/dav/Contacts/',
    'caldav': '/dav/Calendar/',
}


class Root(PathObject, Protocol):
    __pattern__ = '.well-known'
    __namespace__ = None
    __xmlrpc__ = False

    def __init__(self, parent, **params):
        PathObject.__init__(self, parent, **params)

    def is_public(self):
        return True

    def get_name(self):
        return '.well_known'

    def object_for_key(self, name, auto_load_enabled=True, is_webdav=False):
        if name in WELL_KNOWN_REDIRECTS:
            return RedirectObject(
                self, name,
                context=self.context,
                request=self.request,
                parameters=self.parameters,
                location=WELL_KNOWN_REDIRECTS[name],
            )
        elif name == 'opengroupware':
            return OpenGroupwareFolder(
                self, name,
                context=self.context,
                request=self.request,
                parameters=self.parameters,
                location='',
            )
        else:
            raise NotImplementedException(
                'No well-known for "{0}" implemented at this time.'
                .format(name, )
            )
